//
//  LibytnefHelper.m
//  TNEFsEnough
//
//  Created by Josh Jacob on 10/19/11.
//  Copyright 2011 Josh Jacob. All rights reserved.
//

#import "LibytnefHelper.h"
#import "ytnef.h"
#import "PrintTNEF.h"
#import "LibytnefFileItem.h"
#import "LibytnefNestedTaskItem.h"
#import "LibytnefNestedAppointmentItem.h"
#import "LibytnefNestedContactItem.h"
#import "mapidefs.h"

@implementation LibytnefHelper

@synthesize parseResult;
@synthesize from;
@synthesize subject;
@synthesize date;
@synthesize body;
@synthesize bodyRTF;
@synthesize bodyHTML;
@synthesize bodyIsHTML;
@synthesize fileItems;

- (id) initWithFile: (NSURL *) file
{
    self = [super init];
    if (self) {
        
        cStringPath = nil;
        bodyIsHTML = NO;
        fileItems = [[NSMutableArray alloc] initWithCapacity:3];
        
        // get file path as c string
        NSString * pathString = [file path];
        cStringPath = malloc(sizeof(char) * [pathString length] + 2);
        [pathString getCString:cStringPath maxLength:([pathString length] + 2) encoding:NSUTF8StringEncoding];
        
        // init tnef structure
        TNEFInitialize(&tnefStruct);
//        #ifdef DEBUG
        tnefStruct.Debug = 2;
//        #endif
        
        [self parse];
    }
    
    return self;
}

- (void) dealloc
{
    // ytnef
    TNEFFree(&tnefStruct);
    
    // free
    if (cStringPath != nil)
        free(cStringPath);
    
    // release
    
}

#pragma mark - private methods

- (void) parse
{
    // parse file
    self.parseResult = TNEFParseFile(cStringPath, &tnefStruct);
    if (self.parseResult < 0)
    {
        // set error objects and return
        
        return;
    }
    
    // set from/subject/date fields
    if (tnefStruct.from.size > 0) {
        from = [[NSString alloc] initWithCString: tnefStruct.from.data encoding:NSWindowsCP1252StringEncoding];
    } else {
        variableLength * hasSender = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_UNICODE, PR_SENDER_EMAIL_ADDRESS));
        if (hasSender != MAPI_UNDEFINED) {
            from = [[NSString alloc] initWithCString: hasSender->data encoding:NSWindowsCP1252StringEncoding];
        }
    }
    if (tnefStruct.subject.size > 0) {
        subject = [[NSString alloc] initWithCString: tnefStruct.subject.data encoding:NSWindowsCP1252StringEncoding];
    }
    if (tnefStruct.dateSent.wYear > 0) {
        NSDateComponents * comps = [[NSDateComponents alloc] init];
        [comps setHour:tnefStruct.dateSent.wHour];
        [comps setMinute:tnefStruct.dateSent.wMinute];
        [comps setSecond:tnefStruct.dateSent.wSecond];
        [comps setDay:tnefStruct.dateSent.wDay];
        [comps setMonth:tnefStruct.dateSent.wMonth];
        [comps setYear:tnefStruct.dateSent.wYear];
        NSCalendar *gregorian = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
        self.date = [gregorian dateFromComponents:comps];
    }
    
    // set body
    variableLength * hasBody = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_BINARY, PR_RTF_COMPRESSED));
    if (hasBody != MAPI_UNDEFINED) {
        if (IsCompressedRTF(hasBody) == 1)
        {
            int size;
            unsigned char * rawBody = DecompressRTF(hasBody, &size);
            
            NSData *data;
            
            data = [[NSData alloc] initWithBytes:rawBody length:size];
            // NSMutableDictionary * dict = [[NSMutableDictionary alloc] init];
            // [dict setValue:[NSNumber numberWithInt:NSWindowsCP1252StringEncoding] forKey:NSCharacterEncodingDocumentAttribute];
            
#ifdef __MAC_OS_X_VERSION_MAX_ALLOWED
            body = [[NSAttributedString alloc] initWithRTF:data documentAttributes: nil];
#endif
#ifndef __MAC_OS_X_VERSION_MAX_ALLOWED
            bodyRTF = [[NSData alloc] initWithData:data];
#endif
            
            free(rawBody);
            
        } else {
            
        }
    } else {
        
        variableLength * hasBody = MAPIFindProperty(&tnefStruct.MapiProperties, PROP_TAG(PT_BINARY, PR_BODY_HTML));
        if (hasBody != MAPI_UNDEFINED) {
            
            bodyIsHTML = YES;
            NSData *data;
            data = [[NSData alloc] initWithBytes:hasBody->data length:hasBody->size];
            
            NSString * htmlString = [[NSString alloc] initWithData:data encoding:NSISOLatin1StringEncoding];
            self.bodyHTML = htmlString;
            
        }
    }
    
    // get embedded files
    
    Attachment *p = tnefStruct.starting_attach.next;
    while (p != NULL) {
        
        int i; //, object;
        char ifilename[256];
        variableLength *filename;
        //        variableLength *filedata;
        
        if  (p->RenderData.atyp == 1)
        {
            // case 1: printf("File      "); break;
            
            //            if (p->Title.size > 0) 
            //                DLog("%s", p->Title.data);
            //            DLog("\n");
            
            //            if (p->RenderData.dwFlags == 0x00000001) 
            //                printf("     MAC Binary Encoding\n");
            //            if (p->TransportFilename.size >0) 
            //                printf("     Transported under the name %s\n", p->TransportFilename.data);
            
            //            if (p->Date.wYear >0 ) {
            //                printf("    Date: ");
            //                TNEFPrintDate(p->Date);
            //                printf("\n");
            //            }
            //            if (p->CreateDate.wYear > 0) {
            //                printf("    Creation Date: ");
            //                TNEFPrintDate(p->CreateDate);
            //                printf("\n");
            //            }
            //            if (p->ModifyDate.wYear > 0) {
            //                printf("    Modified on: ");
            //                TNEFPrintDate(p->ModifyDate);
            //                printf("\n");
            //            }
            
            //            if (p->MAPI.count>0) {
            //                printf("    MAPI Properties: %i\n", p->MAPI.count);
            //                //if (verbose == 1) {
            //                //    MAPIPrint(&p->MAPI);
            //                //}
            //            }
            
            if (p->FileData.size > 0) {
                // DLog("    Attachment Size:  %ib\n", p->FileData.size);
                
                // default filename
                NSString *nsFileName = @"Untitled Attachment";
                
                // look for ASCII filename
                if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(30,0x3707))) == MAPI_UNDEFINED) {
                    if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(30,0x3001))) == MAPI_UNDEFINED) {
                            filename = &(p->Title);
                    }
                }
                if (filename->size > 0) {
                    sprintf(ifilename, "%s", filename->data);
                    nsFileName = [[NSString alloc] initWithCString:ifilename encoding:NSWindowsCP1252StringEncoding];
                }
            
                // look for unicode filename
                filename = nil;
                if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(31,0x3707))) == MAPI_UNDEFINED) {
                    if ((filename = MAPIFindProperty(&(p->MAPI), PROP_TAG(31,0x3001))) == MAPI_UNDEFINED) {
                        filename = nil;
                    }
                }
                if ((filename != nil) && (filename->size > 0)) {
                    sprintf(ifilename, "%s", filename-> data);
                    nsFileName = [[NSString alloc] initWithCString:ifilename encoding:NSUTF8StringEncoding];
                }
                
                // create item
                LibytnefFileItem * fileItem = [[LibytnefFileItem alloc] initWithAttachment:p fileName:nsFileName size:p->FileData.size];
                
                // add to file list
                [self.fileItems addObject: fileItem];
                
            } else {
                if (p->MAPI.count>0) {
                    MAPIProperty *mp;
                    mp = p->MAPI.properties;
                    for(i = 0; i < p->MAPI.count; i++) {
                        if (mp->nestedMessage != NULL) {
                            if (mp->nestedMessage->messageClass[0] != 0)  {
                                if (strcmp(mp->nestedMessage->messageClass, "IPM.Contact") == 0) {
                                    
                                    LibytnefNestedContactItem *contact = [[LibytnefNestedContactItem alloc] initWithNestedMessage:mp->nestedMessage];
                                    [self.fileItems addObject: contact];
                                    
                                    // NSLog(@"Contact");
                                    // SaveVCard(TNEF );
                                }
                                if (strcmp(mp->nestedMessage->messageClass, "IPM.Task") == 0) {
                                    
                                    LibytnefNestedTaskItem *task = [[LibytnefNestedTaskItem alloc] initWithNestedMessage:mp->nestedMessage];
                                    [self.fileItems addObject: task];
                                    
                                    // NSLog(@"Task");
                                    // SaveVTask(TNEF);
                                }
                                if ((strcmp(mp->nestedMessage->messageClass, "IPM.Appointment") == 0) ||
                                    (strcmp(mp->nestedMessage->messageClass, "IPM.Microsoft Schedule.MtgReq") == 0)) {
                                    
                                    LibytnefNestedAppointmentItem *appointment = [[LibytnefNestedAppointmentItem alloc] initWithNestedMessage:mp->nestedMessage];
                                    [self.fileItems addObject: appointment];
                                    
                                    // NSLog(@"Appointment");
                                    // SaveVCalendar(TNEF);
                                    // foundCal = 1;
                                } else {
                                    NSLog(@"messageClass = %s", mp->nestedMessage->messageClass);
                                }
                            }
                        }
                        mp++;
                    }
                }
            }
        }
        p=p->next;
    }
    
    // test this message as having a different message class
    if (strcmp(tnefStruct.messageClass, "IPM.Contact") == 0) {
        LibytnefNestedContactItem *contact = [[LibytnefNestedContactItem alloc] initWithNestedMessage:&tnefStruct];
        [self.fileItems addObject: contact];
    } else if (strcmp(tnefStruct.messageClass, "IPM.Task") == 0) {
        LibytnefNestedTaskItem *task = [[LibytnefNestedTaskItem alloc] initWithNestedMessage:&tnefStruct];
        [self.fileItems addObject: task];
    } else if ((strcmp(tnefStruct.messageClass, "IPM.Appointment") == 0) ||
        (strcmp(tnefStruct.messageClass, "IPM.Microsoft Schedule.MtgReq") == 0)) {        
        LibytnefNestedAppointmentItem *appointment = [[LibytnefNestedAppointmentItem alloc] initWithNestedMessage:&tnefStruct];
        [self.fileItems addObject: appointment];
    } else {
        NSLog(@"messageClass = %s", tnefStruct.messageClass);
    }
    
    #ifdef DEBUG
    PrintTNEF(tnefStruct);
    #endif
}

@end
