//
//  TNEFDataService.m
//  iTNEFsEnough
//
//  Created by Josh Jacob on 3/9/13.
//  Copyright (c) 2013 Josh Jacob. All rights reserved.
//

#import "TNEFDataService.h"
#import "AppDelegate.h"
#import "TNEFFileModel.h"

@implementation TNEFDataService

+ (NSArray *)readAllFiles
{
    CFAbsoluteTime startTime = CFAbsoluteTimeGetCurrent();
    
    // get managedObjectContext from app
    NSManagedObjectContext * managedObjectContext = [AppDelegate moc];
    
    // build fetch request
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"File" inManagedObjectContext:managedObjectContext];
    [request setEntity:entity];
    
    // set sort descriptor
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"dateAdded" ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    [request setSortDescriptors:sortDescriptors];
    
    // make request
    NSError *error;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    if (mutableFetchResults == nil) {
        DLog(@"Error! %@", [error localizedDescription]);
    }
    
    CFAbsoluteTime endTime = CFAbsoluteTimeGetCurrent();
    DLog(@"getAllFiles - time = %f", (endTime - startTime));
    
    return [NSArray arrayWithArray:mutableFetchResults];
}

+ (TNEFFileModel *)createFileWithName:(NSString *)fileName
                 fromEmail:(NSString *)from
                   subject:(NSString *)subject
                  dateSent:(NSDate *)dateSent
                bodyIsHTML:(BOOL)isHTML
{
    TNEFFileModel *favorite = [NSEntityDescription insertNewObjectForEntityForName:@"File" inManagedObjectContext:[AppDelegate moc]];
    
    favorite.fileName = fileName;
    favorite.from = from;
    favorite.subject = subject;
    favorite.dateSent = dateSent;
    favorite.dateAdded = [NSDate date];
    favorite.bodyIsHTML = [NSNumber numberWithBool:isHTML];
    
    CFUUIDRef uuidObj = CFUUIDCreate(nil);
	NSString* newUUID = (NSString*)CFBridgingRelease(CFUUIDCreateString(nil, uuidObj));
    favorite.uuid = newUUID;
    
    [[AppDelegate moc] save:nil];
    
    return favorite;
}

+ (void)deleteFile:(TNEFFileModel *)file
{
    NSString *saveDir = [NSHomeDirectory() stringByAppendingPathComponent:[NSString stringWithFormat:@"%@/%@/", TNEF_PATH_FILES, file.uuid]];
    NSFileManager *fileMgr = [NSFileManager defaultManager];
    NSError *error = nil;
    [fileMgr removeItemAtPath:saveDir error:&error];
    if (error != nil) {
        DLog(@"Error deleting file: %@", error);
    }
    
    [[AppDelegate moc] deleteObject:file];
    
    [[AppDelegate moc] save:nil];
    
}

+ (void)debugFile:(TNEFFileModel *)file
{
    NSMutableString *debugString = [[NSMutableString alloc] init];
    
    [debugString appendFormat:@"fileName: %@; ", file.fileName];
    [debugString appendFormat:@"dateAdded: %@; ", file.dateAdded];
    [debugString appendFormat:@"dateSent: %@; ", file.dateSent];
    [debugString appendFormat:@"from: %@; ", file.from];
    [debugString appendFormat:@"subject: %@; ", file.subject];
    [debugString appendFormat:@"uuid: %@; ", file.uuid];
    
    DLog(@"%@", debugString);
}

+ (NSArray *)getFiles:(TNEFFileModel *)file
{
    NSMutableArray *files = [[NSMutableArray alloc] init];
    NSString *saveDir = [NSHomeDirectory() stringByAppendingPathComponent:[NSString stringWithFormat:@"%@/%@/", TNEF_PATH_FILES, file.uuid]];
    
    NSError *error;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSArray *fileList = [fileManager contentsOfDirectoryAtPath:saveDir error:&error];
    for (NSString *file in fileList)
    {
        NSString *filePath = [saveDir stringByAppendingPathComponent:file];
        BOOL isDir = NO;
        [fileManager fileExistsAtPath:filePath isDirectory:(&isDir)];
        if (isDir && [file hasPrefix:@"file"])
        {
            NSArray *subDirFileList = [fileManager contentsOfDirectoryAtPath:filePath error:&error];
            for (NSString *subDirFile in subDirFileList)
            {
                [files addObject:[filePath stringByAppendingPathComponent:subDirFile]];
            }
        }
    }
    
    return files;
}

@end
