//
//  MasterViewController.m
//  iTNEFsEnough
//
//  Created by Josh Jacob on 3/22/12.
//  Copyright (c) 2012 Josh Jacob. All rights reserved.
//

#import "MasterViewController.h"

#import "DetailViewController.h"
#import "TNEFDataService.h"
#import "AppDelegate.h"

@interface MasterViewController()

@property (strong) TNEFFileModel *currentFile;

@property (strong) UIBarButtonItem *barItemHide;

@property (strong) UIBarButtonItem *barItemEdit;

@end

@implementation MasterViewController

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Files", @"Files");
    
    [self setClearsSelectionOnViewWillAppear:NO];
    
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(notifyUpdatedFiles)
                                                 name:NOTIFICATION_UPDATED_FILES
                                               object:appDelegate];
    
    if (self.splitViewController.delegate == nil) {
        self.splitViewController.delegate = self;
    }
    
    self.barItemHide = [[UIBarButtonItem alloc] initWithTitle:@"Hide"
                                                        style:UIBarButtonItemStylePlain
                                                       target:self
                                                       action:@selector(hideController:)];
    if (self.splitViewController.displayMode == UISplitViewControllerDisplayModePrimaryOverlay) {
        self.navigationItem.leftBarButtonItem = self.barItemHide;
        self.navigationItem.leftBarButtonItem.enabled = YES;
    }
    
    self.barItemEdit = [[UIBarButtonItem alloc] initWithTitle:@"Edit"
                                                        style:UIBarButtonItemStylePlain
                                                       target:self
                                                       action:@selector(editTable:)];
    self.navigationItem.rightBarButtonItem = self.barItemEdit;
    self.navigationItem.rightBarButtonItem.enabled = YES;
    
//    if (self.splitViewController.displayMode == UISplitViewControllerDisplayModePrimaryOverlay) {
////        NSMutableArray *toolbarButtons = [self.navigationItem.leftBarButtonItems mutableCopy];
////        [toolbarButtons addObject:self.barButtonClose];
////        [self.navigationItem setLeftBarButtonItems:toolbarButtons animated:YES];
//    } else {
//        NSMutableArray *toolbarButtons = [self.navigationItem.leftBarButtonItems mutableCopy];
//        [toolbarButtons removeObject:self.barButtonClose];
//        [self.navigationItem setLeftBarButtonItems:toolbarButtons animated:YES];
//    }
}

//- (void)viewDidUnload
//{
//    [super viewDidUnload];
//}
//
//- (void)viewWillAppear:(BOOL)animated
//{
//    [super viewWillAppear:animated];
//}
//
//- (void)viewDidAppear:(BOOL)animated
//{
//    [super viewDidAppear:animated];
//}
//
//- (void)viewWillDisappear:(BOOL)animated
//{
//	[super viewWillDisappear:animated];
//}
//
//- (void)viewDidDisappear:(BOOL)animated
//{
//	[super viewDidDisappear:animated];
//}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:TNEF_DIR_FILES
                                                  object:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

-(UIInterfaceOrientationMask)supportedInterfaceOrientations
{
    return UIInterfaceOrientationMaskAll;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"Show Detail"]) {
        
        UINavigationController *navVC = [segue destinationViewController];
        for (UIViewController *vc in [navVC childViewControllers]) {
            
            if ([vc isKindOfClass:[DetailViewController class]]) {
                
                // Get destination view
                DetailViewController *detailVC = (DetailViewController *) vc;
                
                detailVC.tnefFile = (TNEFFileModel *)sender;
                // DLog(@"Selected TNEF file %@", detailVC.tnefFile.fileName);
                
            }
            
        }
    }
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    if (appDelegate.files.count == 0)
        return 0;
    else
        return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    return appDelegate.files.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
    }

    // Get data object
    TNEFFileModel *file = appDelegate.files[[indexPath indexAtPosition:1]];
    
    // Configure the cell.
    cell.textLabel.text = file.subject;
    if (file.subject == nil || file.subject.length == 0) {
        cell.textLabel.text = @"<no subject>";
    }
    cell.detailTextLabel.text = [NSString stringWithFormat:@"From: %@", file.from];
    
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

#pragma mark - UITableViewDelegate

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return UITableViewCellEditingStyleDelete;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
        
        TNEFFileModel *file = [appDelegate.files objectAtIndex:[indexPath indexAtPosition:1]];
        
        // reset if we deleted the one in the detail view
        if (file == _currentFile) {
            [self performSegueWithIdentifier:@"Show Detail" sender:nil];
        }
        
        [appDelegate.files removeObjectAtIndex:[indexPath indexAtPosition:1]];
        [TNEFDataService deleteFile:file];

        [self.tableView reloadData];
        
//        if (![self.tableView isEditing]) {
//            [self performSegueWithIdentifier:@"Show Detail" sender:nil];
//        }
        
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if ([self.tableView isEditing]) {
        return;
    }
    
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    
    NSUInteger index = [indexPath indexAtPosition:1];
    TNEFFileModel *file = appDelegate.files[index];
    _currentFile = file;
    
    [self performSegueWithIdentifier:@"Show Detail" sender:file];
}

#pragma mark - UISplitViewControllerDelegate

- (void)splitViewController:(UISplitViewController *)svc
    willChangeToDisplayMode:(UISplitViewControllerDisplayMode)displayMode
{
    // DLog(@"displayMode = %ld", (long)displayMode);
    
    if (displayMode == UISplitViewControllerDisplayModePrimaryOverlay) {
        
        // [self.navigationController.toolbar setItems:@[self.barItemHide] animated:YES];
        
        // [self.navigationController.navigationBar setItems:@[self.barItemHide] animated:YES];
        
        self.navigationItem.leftBarButtonItem = self.barItemHide;
        self.navigationItem.leftBarButtonItem.enabled = YES;
        
//        NSMutableArray *toolbarButtons = [self.navigationItem.leftBarButtonItems mutableCopy];
//        [toolbarButtons addObject:self.barButtonClose];
//        [self.navigationItem setLeftBarButtonItems:toolbarButtons animated:YES];
    } else {
        
        // [self.navigationController.toolbar setItems:@[] animated:YES];
        
        // [self.navigationController.navigationBar setItems:@[] animated:YES];
        
        self.navigationItem.leftBarButtonItem = nil;
        self.navigationItem.leftBarButtonItem.enabled = YES;
        
//        NSMutableArray *toolbarButtons = [self.navigationItem.leftBarButtonItems mutableCopy];
//        [toolbarButtons removeObject:self.barButtonClose];
//        [self.navigationItem setLeftBarButtonItems:toolbarButtons animated:YES];
    }
}

#pragma mark - NSNotification

-(void) notifyUpdatedFiles
{
    DLog(@"New files notification");
    
    [self.tableView reloadData];
    
    [self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]
                                animated:YES
                          scrollPosition:UITableViewScrollPositionTop];
    
    AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
    TNEFFileModel *file = appDelegate.files[0];
    _currentFile = file;
    [self performSegueWithIdentifier:@"Show Detail" sender:file];
}

#pragma mark - IBActions

- (IBAction)hideController:(id)sender {
    
    //    [self.splitViewController.displayModeButtonItem.target
    //        performSelector:self.splitViewController.displayModeButtonItem.action
    //        withObject: self.splitViewController.displayModeButtonItem];
    
    [UIView animateWithDuration:0.2 animations:^{
        self.splitViewController.preferredDisplayMode = UISplitViewControllerDisplayModePrimaryHidden;
    } completion:^(BOOL finished) {
        self.splitViewController.preferredDisplayMode = UISplitViewControllerDisplayModeAutomatic;
    }];
    
}

- (IBAction)editTable:(id)sender {
    
    if ([self.tableView isEditing]) {
        [self.tableView setEditing:NO animated:YES];
        self.barItemEdit.title = @"Edit";
        
        // [self.tableView reloadData];
//        AppDelegate *appDelegate = (AppDelegate *) [[UIApplication sharedApplication] delegate];
//        if (appDelegate.files.count > 0) {
//            [self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]
//                                        animated:YES
//                                  scrollPosition:UITableViewScrollPositionTop];
//        }
        // [self performSegueWithIdentifier:@"Show Detail" sender:nil];
        
    } else {
        [self.tableView setEditing:YES animated:YES];
        self.barItemEdit.title = @"Done";
    }
}

@end
